/*
Weight function of the CET (continuous Euler transformation):
w^L(x) = cet_wish(beta, 2*beta*x/L-beta),
	beta = -log(eps), 
	L = pi*beta/omega_min.
	eps       : Relative error requested
	omega_min : Minimum frequency of exp(i*omega*x)

CET approximation formula:
	Integral[0,infinity] f(x)*exp(i*omega*x) dx
	= Integral[0,L] w^L(x)*f(x)*exp(i*omega*x) dx + O(eps)

Ref.
[1] T. Ooura, 
	"A continuous Euler transformation and its application to the 
	Fourier transform of a slowly decaying function", 
	J. Comput. Appl. Math., 130 (2001) 259-270.
[2] T. Ooura, 
	"Numerical inversion of the Laplace transform using a 
	continuous Euler transformation"
	RIMS Kokyuroku, Kyoto Univ., 1145, (2000), 188-193.
*/

#include <float.h>


double cet_ishs(double b2, double x2)
{
	int n;
	double p = 1, q = 1, s = 1;
	for (n = 2; q >= DBL_EPSILON; n += 2) {
		p = x2 * p / (n * (n - 1));
		q = (b2 * q + p) / ((n + 1) * (n + 1));
		s += q;
	}
	return s;
} /* 1/x * integral[0,x] sinh(sqrt(b2-t^2))/sqrt(b2-t^2) dt, x=sqrt(b2-x2) */


double cet_wish(double beta, double x)
{
	static double beta0 = 0, cnorm = 0;
	if (x < -beta) return 1; /* illegal argument */
	if (x > beta) return 0; /* illegal argument */
	if (beta != beta0) {
		cnorm = 1 / (2 * beta * cet_ishs(beta * beta, 0));
		beta0 = beta;
	}
	return 0.5 - cnorm * (x * cet_ishs(beta * beta, beta * beta - x * x));
} /* 1/2 - cnorm * integral[0,x] sinh(sqrt(beta^2-t^2))/sqrt(beta^2-t^2) dt */

